<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Dto\Classes\ResetPasswordCodeDto;
use App\Service\Dto\DtoServiceInterface;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

class ResetPasswordCode extends BaseActionService
{

    public function __construct(
        private ResponseServiceInterface    $response,
        private UserPasswordHasherInterface $passwordHasher,
        private DtoServiceInterface         $passwordResetDto,
        private ManagerRegistry             $doctrine
    )
    {
        parent::__construct($response);
    }

    public function runAction(): void
    {
        /** @var ResetPasswordCodeDto $dto */
        $dto = $this->passwordResetDto->getClass();
        /** @var User $userExists */
        $userExists = $this->doctrine->getRepository(User::class)
            ->findOneByUniq($dto->email, $dto->phoneNumber);

        if ($userExists !== null) {
            $currentDate = new \DateTime();
            $code = $dto->code;
            $registerCode = $userExists->getRegisterCode();
            if ($registerCode === null) {
                $this->response->addError('Код подтверждения не отправлен');
            } else {
                if ($registerCodeDate = $registerCode->getDate()) {
                    if ($registerCode->getCode() === $code && $currentDate->getTimestamp() < $registerCodeDate->getTimestamp()) {
                        try {
                            $hashedPassword = $this->passwordHasher->hashPassword(
                                $userExists,
                                $dto->password ?: ''
                            );
                            $userExists->setPassword($hashedPassword);
                            $em = $this->doctrine->getManager();
                            $em->persist($userExists);
                            $em->remove($registerCode);
                            $em->flush();
                            $this->response->addMessage('Пароль изменен');
                        } catch (\Exception $exception) {
                            $this->response->addError('Ошибка изменения пароля');
                        }
                    } else {
                        $this->response->addError('Код недействителен');
                    }
                } else {
                    $this->response->addError('Код недействителен');
                }
            }
        } else {
            $this->response->addError('Пользователь не найден');
        }
    }

    public function validate(): bool
    {
        return $this->passwordResetDto->validate($this->response);
    }
}